{% include 'header.jinja2' %}

{% if component.taxonomy_type == 'CONTENT' %}
{% include 'taxonomy-structure.jinja2' %}
{% endif %}

{% if component.taxonomy_type == 'PROCESSING' or component.taxonomy_type == 'MODEL' %}
{% include 'taxonomy-labels.jinja2' %}
{% endif %}